from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_marshmallow import Marshmallow
{%- if celery %}
from celery import Celery
{%- endif %}

from hobbit_core import HobbitManager

db = SQLAlchemy()
migrate = Migrate()
ma = Marshmallow()
{%- if celery %}
celery = Celery()
{%- endif %}
hobbit = HobbitManager()

