image: ttwshell/hobbit-app:latest

test_all:
  variables:
    FLASK_APP: 'app/run.py'
    FLASK_ENV: 'testing'
    LANG: C.UTF-8
  script:
    - flake8 . --max-line-length=120 --exclude migrations
    - py.test --cov . --cov-report term-missing -s -x

staging-dev:
  type: deploy
  script:
    - ./deploy.sh 127.0.0.1
  only:
    - master

staging-test:
  type: deploy
  script:
    - ./deploy.sh 127.0.0.1
  only:
    - tags
